;;;{{{DOC
;;; -*- Mode: LISP; Package: CYC; Syntax: ANSI-Common-Lisp -*-
;;;
;;; Copyright (c) 2002 - 2010 Douglas R. Miles.  All rights reserved.
;;;
;;; @module COGBOT-PARSER-TESTS
;;; @features :COGBOT-NL
;;;
;;; @author dmiles
;;; @owner dmiles
;;;
;;; @created 2010/01/10
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; tools & utilities for Cogbot/NLParsing
;;
;; This is to test how the Cogbot parser is doing
;;
;; !!!!!!!!!!!!!!!
;; IMPORTANT NOTES:
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; External interface:
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Depends on interface: COGBOT-PARSER
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;}}}EDOC

(define test-e2c (name str)
  (clet ((res (logicmoo-rkf-reader str #$STemplate)))
    (punless res
       (progn
	  (csetq res (parse str))
	  (pwhen res (ret (print `(:MAYBE ,str))))
          (ret (print `(:FAIL ,str)))))
    (ret (print `(:SUCCESS ,str ,res)))))

(test-e2c 0 "George fell last year")
(test-e2c 0 "Oil field revenues fell last year")
(test-e2c 0 "I see you")
(test-e2c 0 "I seen the wood")
(test-e2c 0 "I sawed the wood")
(test-e2c 0 "I see two books on a shelf")
(test-e2c 1 "I see two books sitting on a shelf")
(test-e2c 0 "I can see you")
(test-e2c 0 "Fido is a dog")
(test-e2c 0 "Who is the president of the United States of America?")

(test-e2c 1 "I saw him")
(test-e2c 1 "I saw him Monday")
(test-e2c 2 "I saw him January")
(test-e2c 1 "I saw him January twenty first")
(test-e2c 1 "Monday's concert should be good")
(test-e2c 2 "January 21, I saw him")
(test-e2c 1 "This Monday's concert should be good")
(test-e2c 1 "This week's concert should be good")
(test-e2c 1 "The Monday concert should be good")
(test-e2c 1 "We did it on Jan 21")
(test-e2c 2 "We did it in Jan")
(test-e2c 2 "I saw him on early January 21")
(test-e2c 2 "The this week concert should be good")
(test-e2c 1 "I saw him in January 1990")

;; ( PRINT "NOUNS")

(test-e2c 1 "The fact that he smiled at me gives me hope")
(test-e2c 2 "The event that he smiled at me gives me hope")
(test-e2c 1 "But my efforts to win his heart have failed")
(test-e2c 2 "But my presents to win his heart have failed")
(test-e2c 1 "Failure to comply may result in dismissal")
(test-e2c 2 "Absence to comply may result in dismissal")
(test-e2c 1 "The question is who we should invite")
(test-e2c 2 "The party is who we should invite")
(test-e2c 1 "The big question on everybody's mind is who killed OJ")
(test-e2c 2 "The big mind on everybody's question is who killed OJ")
(test-e2c 1 "Do it the way you've always done it")
(test-e2c 1 "I really like the way you do your hair")
(test-e2c 2 "I really like the fashion you do your hair")
(test-e2c 1 "He made a request that there be an investigation")
(test-e2c 2 "He stated the fact that there be an investigation")
(test-e2c 1 "He is the kind of person who would do that")
(test-e2c 2 "He is the character of person who would do that")
(test-e2c 1 "An income tax increase may be necessary")
(test-e2c 2 "A tax on income increase may be necessary")
(test-e2c 1 "Last week I saw a great movie")
(test-e2c 2 "Last dog I saw a great movie")
   /*
(test-e2c 1 "The party that night was a big success")
(test-e2c 2 "The party that dog was a big success")
(test-e2c 1 "John Stuart Mill is an important author")
(test-e2c 1 "The Richard Milhous Nixon Library has been a big success")
(test-e2c 1 "The mystery of the Nixon tapes was never solved")
(test-e2c 1 "High income taxes are important")
(test-e2c 1 "Oil company stock prices rose in heavy trading today")
(test-e2c 1 "Metals futures prices rose in heavy trading today")
(test-e2c 1 "U.S. economic indicators fell sharply last month")      
(test-e2c 1 "Columbia medical and administrative workers continued their strike today")
(test-e2c 1 "Janet, who is an expert on dogs, helped me choose one")
(test-e2c 2 "Janet who is an expert on dogs helped me choose one")
(test-e2c 1 "The dog that we eventually bought was very expensive")
(test-e2c 2 "The dog, that we eventually bought, was very expensive")
(test-e2c 2 "The dog, we eventually bought, was very expensive")
(test-e2c 1 "Have you ever seen the Pacific")
(test-e2c 1 "The new David Letterman is a happy, relaxed David Letterman")
(test-e2c 1 "Actress Whoopi Goldberg and singer Michael Jackson attended the ceremony")
(test-e2c 1 "We are from the planet Gorpon")
(test-e2c 1 "This is my friend Bob")
(test-e2c 1 "John's family is renovating their kitchen")
(test-e2c 2 "A man I know's family is renovating their kitchen")
(test-e2c 1 "The boys' bedrooms will be enlarged")
(test-e2c 2 "The boys's bedrooms will be enlarged")
(test-e2c 1 "My uncle's mother's cousin is visiting us")
(test-e2c 2 "Emily's my cousin is visiting us")
(test-e2c 1 "We ate at Joe's Diner last week")
(test-e2c 1 "The buy-out caused a free-for-all in the mid-afternoon")
(test-e2c 1 "Joan Smith is president of the company")
(test-e2c 2 "Joan Smith is tourist")
(test-e2c 1 "Alfred Baird, formerly vice president of Beevil Corp., has been appointed as president")*/
(test-e2c 2 "Alfred Baird, man I know, has been described as good gardener")
(test-e2c 1 "A bunch of dogs are in the yard")
(test-e2c 2 "A picture of dogs are in the yard")

;; ( PRINT "DETERMINERS & PRONOUNS")

(test-e2c 1 "Many people were angered by the hearings")
(test-e2c 2 "Many person were angered by the hearings")
(test-e2c 1 "Many were angered by the hearings")
(test-e2c 1 "My many female friends were angered by the hearings")
(test-e2c 2 "My some female friends were angered by the hearings")
(test-e2c 1 "Many who initially supported Thomas later changed their minds")
(test-e2c 1 "The stupidity of the senators annoyed all my friends")
(test-e2c 2 "The stupidity of the senators annoyed many my friends")
(test-e2c 1 "I need to buy a present, but I want something inexpensive")
(test-e2c 2 "I need to buy a present, but I want a gift inexpensive")
(test-e2c 1 "Anyone who thinks this will work is crazy")
(test-e2c 1 "Their program is better than ours")
(test-e2c 1 "Those that want to come can come")
(test-e2c 1 "I read everything I could about the subject")
(test-e2c 1 "I read whatever I could about the subjetc")
(test-e2c 2 "I read several books I could about the subject")
(test-e2c 1 "the best costumes got prizes")
(test-e2c 2 "a best costume got prizes")
(test-e2c 2 "some best costumes got prizes")
(test-e2c 1 "the best five costumes got prizes")
(test-e2c 1 "the five best costumes got prizes")
(test-e2c 2 "the five best five costumes got prizes")
(test-e2c 2 "the hundreds of best costumes got prizes")
(test-e2c 2 "five best costumes got prizes")
(test-e2c 2 "best costumes got prizes")
(test-e2c 2 "best five costumes got prizes")
(test-e2c 1 "five other costumes got prizes")
(test-e2c 1 "the other five costumes got prizes")
(test-e2c 2 "other five costumes got prizes")
(test-e2c 1 "the other candy was really disgusting")
(test-e2c 2 "the five other candy was really disgusting")
(test-e2c 1 "other candy is really disgusting")
(test-e2c 2 "other piece of candy was really bad")
(test-e2c 1 "other costumes were really bad")
(test-e2c 1 "some other costumes were really bad")

;; ( PRINT "TIME EXPRESSIONS")

(test-e2c 1 "We're thinking about going to a movie this evening")
(test-e2c 2 "We're thinking about going to a movie this theater")
(test-e2c 1 "I've been grading these stupid exams all day")
(test-e2c 2 "I've been grading these stupid days all exam")
(test-e2c 1 "We're having a big party Tuesday")
(test-e2c 2 "We're having a big party our house")
(test-e2c 2 "There is going to be an important meeting January")
(test-e2c 1 "There is going to be an important meeting in January")
(test-e2c 1 "There is going to be an important meeting next January")
(test-e2c 1 "The party last week was a big success")
(test-e2c 1 "John last week threw a great party")
(test-e2c 1 "Until recently, these fossils were believed to belong to different species")
(test-e2c 2 "Until initially, these fossils were believed to belong to different species")
(test-e2c 2 "Until for many years, these fossils were believed to belong to different species")
(test-e2c 1 "Until last week, these fossils were believed to belong to different species")
(test-e2c 2 "Until last meeting, these fossils were believed to belong to different species")
(test-e2c 1 "I'm quite excited about next week")
(test-e2c 1 "Monday sounds good for the meeting")
(test-e2c 1 "Tomorrow might be a good time for the meeting")
(test-e2c 1 "Last Tuesday was really fun")
(test-e2c 2 "Three days ago was really fun")
(test-e2c 1 "Almost three years after our first date, I saw Ruth again")
(test-e2c 1 "Almost three years after I first met her, I saw Ruth again")
(test-e2c 2 "Almost three years, I saw Ruth again")
(test-e2c 1 "Almost three years later, I saw Ruth again")
(test-e2c 2 "Almost three years for our first date, I saw Ruth again")
(test-e2c 1 "I saw her again a year and a half later")
(test-e2c 2 "I saw her again a year and a dog later")
(test-e2c 1 "He left here a quarter of an hour ago")
(test-e2c 2 "He left here a quarter of a dog ago")
(test-e2c 2 "He left here a picture of an hour ago")
(test-e2c 1 "I still remember the day I kissed him")
(test-e2c 2 "I still remember the room I kissed him")
(test-e2c 1 "I'm going to Europe the day I graduate")
(test-e2c 1 "Clinton is expected to return to Washington Thursday morning")
(test-e2c 2 "Clinton is expected to return to Thursday Washington office")
(test-e2c 1 "Clinton is expected to return to Washington on Thursday morning")
(test-e2c 1 "Clinton is expected to return to Washington late Thursday morning")
(test-e2c 1 "Clinton is expected to return to Washington next Thursday morning")
(test-e2c 1 "She walked out of the room the minute I saw her")
(test-e2c 2 "She walked out of the room two minutes I saw her")
(test-e2c 1 "I was crazy about him the first time I saw him")
(test-e2c 2 "I was crazy about him the first party I saw him")
(test-e2c 1 "In January 1990, a historic new law was passed")
(test-e2c 2 "In Washington 1990, a historic new law was passed")
(test-e2c 1 "On January 15, 1990, a historic new law was passed")
(test-e2c 2 "On January 320, 1990, a historic new law was passed")
(test-e2c 1 "He was convicted under an obscure 1990 law ")
(test-e2c 2 "He was convicted under an obscure 50 law")
(test-e2c 1 "I wish I could see him 100 times a day")
(test-e2c 2 "I wish I could see him 3 meals a day")
(test-e2c 2 "I wish I could see him 100 times a dog")
(test-e2c 2 "I wish I could see him 100 times 2 days")
(test-e2c 1 "It's good to eat three big meals a day")
(test-e2c 1 "Someone is mugged in New York every five minutes")
(test-e2c 1 "Her career lasted almost thirty years")
(test-e2c 2 "Her career lasted almost thirty books")
(test-e2c 1 "Every morning at 5 a.m., Ken gets up and runs for five miles")
(test-e2c 1 "At 4:52 p.m., on Monday, December 26, 1997, nothing happened")

;; ( PRINT "VERBS ")

(test-e2c 1 "Abrams does like programming")
(test-e2c 2 "Abrams does be a good programmer")
(test-e2c 1 "He is being hired by another company")
(test-e2c 1 "He is looking for another job")
(test-e2c 1 "Fred has had five years of experience as a programmer")
(test-e2c 2 "Fred has had been a programmer for five years")
(test-e2c 1 "I gave my mother the present I bought for her")
(test-e2c 1 "I gave her the present I bought for her")
(test-e2c 2 "I gave my mother it")
(test-e2c 1 "We picked out some beautiful flowers for her")
(test-e2c 1 "We picked some beautiful flowers out for her")
(test-e2c 1 "We picked them out for her")
(test-e2c 2 "We picked out them for her")
(test-e2c 1 "Did you put the milk in the refrigerator")
(test-e2c 2 "Did you put the milk")
(test-e2c 1 "Where did you put the milk")
(test-e2c 1 "I hope he comes to the party tomorrow")
(test-e2c 1 "I hope that he comes to the party tomorrow")
(test-e2c 2 "I hope him to come to the party tomorrow")
(test-e2c 1 "I expect him to come to the party tomorrow")
(test-e2c 1 "I expect to go to the party tomorrow")
(test-e2c 2 "I expect")
(test-e2c 2 "I expected who would come to the party")
(test-e2c 1 "I knew who would come to the party")
(test-e2c 2 "I expected he go to the party")
(test-e2c 1 "I suggested he go to the party")
(test-e2c 2 "He knew me how to use the program")
(test-e2c 1 "He asked me how to use the program")
(test-e2c 2 "He disputed our program was superior")
(test-e2c 1 "He disputed that our program was superior")
(test-e2c 1 "Anne told me I would almost certainly be hired")
(test-e2c 2 "Anne expected me I would almost certainly be hired")
(test-e2c 2 "We argued adding new features to the program")
(test-e2c 1 "We discussed adding new features to the program")
(test-e2c 2 "I thought terrible after our discussion")
(test-e2c 1 "I felt terrible after our discussion")
(test-e2c 1 "I made him make some changes in the program")
(test-e2c 2 "I encouraged him make some changes in the program")
(test-e2c 1 "I helped him make some changes in the program")
(test-e2c 1 "I helped make some changes in the program")
(test-e2c 2 "I saw make some changes in the program")
(test-e2c 2 "I made him telling her about the party")
(test-e2c 1 "I saw him telling her about the party")
(test-e2c 1 "Phil gave me a sweater which he bought in Paris")
(test-e2c 2 "Phil chose me a sweater which he bought in Paris")
(test-e2c 1 "Alan bet me five dollars Clinton would lose the election")
(test-e2c 2 "Alan offered me five dollars Clinton would lose the election")
(test-e2c 1 "She said she didn't approve of my behavior")
(test-e2c 2 "She said she didn't like of my behavior")
(test-e2c 1 "The results are in, the game is up and the truth is out")
(test-e2c 2 "The in results show the out truth about the up game")
(test-e2c 2 "The results became in and the truth seemed out")
(test-e2c 1 "He sold for five dollars the ring his mother had given him")
(test-e2c 1 "Clinton announced on Tuesday a bold new proposal")
(test-e2c 2 "Clinton announced on Tuesday it")
(test-e2c 1 "I gave my brother an expensive present")
(test-e2c 1 "I gave him an expensive present")
(test-e2c 1 "I gave an expensive present")
(test-e2c 1 "I gave it")
(test-e2c 2 "I gave my brother it")
(test-e2c 1 "I gave him for his birthday a very expensive present ")
(test-e2c 2 "I gave him for his birthday it")
(test-e2c 1 "I gave for his birthday an expensive present ")
(test-e2c 2 "I gave for his birthday it")
(test-e2c 1 "The President announced on Monday that several more bases would be closed")
(test-e2c 1 "He had attempted for years to make a career as a concert pianist")
(test-e2c 2 "He had attempted for years")
(test-e2c 1 "I asked him when I saw him at the party yesterday what he was working on")
(test-e2c 2 "I talked to him when I saw him at the party yesterday what he was working on")
(test-e2c 1 "I wondered for a long time why everyone liked her so much")
(test-e2c 2 "I thought for a long time why everyone liked her so much")
(test-e2c 1 "I told Margaret that I thought she would probably be hired")
(test-e2c 2 "I told on Tuesday Margaret that I thought she would probably be hired")
(test-e2c 1 "I told Margaret on Tuesday that I thought she would probably be hired")
(test-e2c 1 "We discussed at the meeting hiring a new secretary")
(test-e2c 2 "We discussed at the meeting")
(test-e2c 1 "We informed the new employees that no salary increase would be possible")
(test-e2c 1 "We informed at the meeting the new employees")
(test-e2c 2 "We informed at the meeting the new employees that no salary increase would be possible")
(test-e2c 1 "They were asked that he be allowed to go")
(test-e2c 1 "If his calculations were correct, Copernicus reasoned, the earth must revolve around the sun")
(test-e2c 1 "The earth, Copernicus reasoned, must revolve around the sun")
(test-e2c 1 "The earth must revolve around the sun, Copernicus reasoned")
(test-e2c 2 "The earth must revolve around the sun, Copernicus was happy")
(test-e2c 2 "The earth must revolve around the sun, Copernicus destroyed")
(test-e2c 2 "The earth, the pope cringed when Copernicus reasoned, revolves around the sun")
(test-e2c 1 "Abortion was legal until the third month, the court ruled")
(test-e2c 1 "If the pregnancy was within the first three months, the court ruled, abortion was legal")
(test-e2c 1 "Nobody, it seems, wants to be a liberal")
(test-e2c 2 "Nobody, John seems, wants to be a liberal")
(test-e2c 1 "Business is booming, Joe Smith, a car dealer, says")
(test-e2c 1 "Business is booming, says Joe Smith, a car dealer")
(test-e2c 1 "You can do anything you want, I told her")
(test-e2c 1 "In the last few years, it seems, nobody wants to be a liberal")
(test-e2c 1 "Also invited to the meeting were several prominent scientists")
(test-e2c 2 "Also invited to the meeting invited several prominent scientists")
(test-e2c 1 "Also awarded the prize was Jean Smith, a prominent computer scientist")
(test-e2c 1 "Chosen to lead the commission was Fred Schultz, a former Federal judge")
(test-e2c 2 "Chosen to lead the commission seemed likely to be Fred Schultz, a former Federal judge")
(test-e2c 2 "Chooses to lead the investigation Fred Schultz")
(test-e2c 2 "Choose to lead the investigation did Fred Schultz")
(test-e2c 1 "Also recommended in the report was a new initiative to combat crime")
(test-e2c 2 "Also chosen the leader for the commission was Fred Schultz")
(test-e2c 1 "Included in our paper is a summary of the features of our program")
(test-e2c 1 "Also performing in the concert were members of the Budapest Quartet")
(test-e2c 2 "Were performing in the concert members of the Budapest Quartet")
(test-e2c 1 "Voting in favor of the bill were 36 Republicans and 4 moderate Democrats")
(test-e2c 1 "Glaring coldly at Sarah, he walked out of the room")
(test-e2c 1 "He walked out of the room, glaring coldly at Sarah")
(test-e2c 2 "Glaring coldly at Sarah, walking out of the room")
(test-e2c 1 "Finding that it was impossible to get work as a waiter, he worked as a janitor")
(test-e2c 1 "He had hoped to get work as a waiter, but, finding this was impossible, he worked as a janitor")
(test-e2c 2 "He said that, finding that it was impossible to get work as a waiter, he would work as a janitor")
(test-e2c 1 "Used by some of the finest pianists in the country, Baldwin pianos are technical marvels")
(test-e2c 1 "Using specially designed parts, Baldwin pianos are technical marvels")
(test-e2c 2 "Used specially designed parts, Baldwin pianos are technical marvels")
(test-e2c 1 "Sending a message of discontent to Washington, voters overwhelmingly rejected the Clinton administration")
(test-e2c 1 "She's a really good player")
(test-e2c 1 "John's coming to the party tonight")
(test-e2c 1 "He's usually gone to Boston for Thanksgiving")
(test-e2c 2 "Do you know where John's")
(test-e2c 1 "Who's afraid of the big bad wolf")
(test-e2c 1 "That's just the kind of person he is")
(test-e2c 2 "That's just the kind of person he's")
(test-e2c 1 "There's no reason to get so upset about it")
(test-e2c 1 "I didn't think he would do it, but he did")
(test-e2c 2 "I didn't think he would invite her, but he invited")
(test-e2c 1 "If you don't want to do it, you should find someone who will")
(test-e2c 1 "If you don't want to do it, you should find someone who does")
(test-e2c 1 "Find someone who does")
(test-e2c 2 "Find someone who wants to do")
(test-e2c 1 "I don't like programming, and someone who does may be difficult to find")
(test-e2c 1 "The price of the stock more than doubled in two days")
(test-e2c 2 "The price of the stock more than increased in two days")
(test-e2c 1 "I finally figured out why this program is so slow")
(test-e2c 2 "I finally flipped out why this program is so slow")
(test-e2c 1 "It turns out that Clinton didn't actually kiss Monica")
(test-e2c 1 "If you go around doing that, you're going to end up making people mad")
(test-e2c 1 "The lawyer pointed out that Clinton didn't actually kiss Monica")
(test-e2c 1 "A party is taking place, so if you'd like to show up, you should do so")

;; ( PRINT "PREPOSITIONS")

(test-e2c 1 "I have doubts about inviting him")
(test-e2c 2 "I have doubts during inviting him")
(test-e2c 1 "I prevented her from doing it by praising her for not doing it")
(test-e2c 1 "From your description, I don't think I would enjoy it")
(test-e2c 1 "We had an argument over whether it was a good movie")
(test-e2c 2 "We had an argument at whether it was a good movie")
(test-e2c 1 "Because of the rain, we decided to stay home")
(test-e2c 1 "They're having a party in front of the building")
(test-e2c 1 "The man with whom I play tennis is here")
(test-e2c 1 "The man I play tennis with is here")
(test-e2c 2 "The man whom I play tennis is here")
(test-e2c 2 "The man with whom I play tennis with is here")
(test-e2c 1 "With whom did you play tennis ")
(test-e2c 1 "Who did you play tennis with ")
(test-e2c 1 "The data on file will be used for the project at hand, which is already under way")
(test-e2c 2 "The data on project will be used for the file at program")
(test-e2c 1 "The project was finished on schedule, as usual")
(test-e2c 2 "The project as usual and the report on schedule were finished ")
(test-e2c 1 "I told him by telephone that I was coming by car")
(test-e2c 1 "From in back of the shed, I heard a scream that seemed to come from on top of the garage")
(test-e2c 1 "The university spends $5 per student")
(test-e2c 2 "The university spends $5 per the student")
(test-e2c 1 "They're building a gigantic mall five miles from here")
(test-e2c 2 "They're building a gigantic mall five mistakes from here")
(test-e2c 1 "The 7-11 is half a mile up the road, but the supermarket is a long way away")
(test-e2c 1 "I'm sure things will get done with Janet running the company ")
(test-e2c 2 "I'm sure things will get done by Janet running the company ")
(test-e2c 2 "I'm sure things will get done because Janet running the company ")
(test-e2c 1 "With Janet in charge, I'm sure things will get done")

;; ( PRINT "ADJECTIVES")

(test-e2c 1 "You are lucky that there is no exam today")
(test-e2c 2 "You are stupid that there is no exam today")
(test-e2c 1 "You are lucky I am here")
(test-e2c 2 "You are right I am here")
(test-e2c 1 "This is something we should be happy about")
(test-e2c 2 "This is something we should be happy ")
(test-e2c 2 "The happy about it man kissed his wife")
(test-e2c 1 "Is he sure how to find the house")
(test-e2c 2 "Is he correct how to find the house")
(test-e2c 1 "You should be proud of your achievement")
(test-e2c 2 "You should be happy of your achievement")
(test-e2c 1 "He is the smartest man I know")
(test-e2c 2 "They are some smartest men I know")
(test-e2c 1 "I've seen a lot of programs, but ours is the fastest")
(test-e2c 1 "Ours is the fastest of the programs we have seen")
(test-e2c 1 "I've seen a lot of programs, but ours runs the most quickly")
(test-e2c 2 "This is our the fastest program")
(test-e2c 1 "Voters angry about the economy will probably vote for Clinton")
(test-e2c 2 "Voters angry will probably vote for Clinton")
(test-e2c 1 "Many Democrats unhappy about the economy but doubtful that Clinton can be elected probably won't vote at all")
(test-e2c 2 "Many Democrats unhappy but doubtful probably won't vote at all")
(test-e2c 2 "Many Democrats likely that Bush will be reelected probably won't vote")
(test-e2c 1 "Hundreds of young men, furious about the verdict in the Rodney King case, looted stores in Los Angeles today")
(test-e2c 2 "Hundreds of young men, furious, looted stores in Los Angeles today")
(test-e2c 1 "We need a programmer knowledgeable about Lisp")
(test-e2c 2 "We need a programmer knowledgeable")
(test-e2c 1 "Any program as good as ours should be useful")
(test-e2c 2 "Any program good should be useful")
(test-e2c 1 "Let us know if you have a program capable of parsing this utterance")
(test-e2c 2 "Let us know if you have a program capable")
(test-e2c 1 "It is believed that even the troops loyal to Hussein will soon be forced to surrender")
(test-e2c 2 "It is believed that even the troops loyal will soon be forced to surrender")
(test-e2c 1 "Republican policies only benefit the rich and powerful")
(test-e2c 1 "Republican policies only benefit the rich and the powerful")
(test-e2c 2 "Republican policies only benefit a rich and a powerful")
(test-e2c 2 "Republican policies only benefit some rich and some powerful")
(test-e2c 1 "The meek will inherit the earth, and the best is the enemy of the good")
(test-e2c 1 "They're building a skyscraper over 1000 feet tall")
(test-e2c 2 "They're building a skyscraper tall")
(test-e2c 1 "The river is half a mile wide here and 300 feet deep")
(test-e2c 2 "The river is half a mile beautiful and 300 feet dangerous")
(test-e2c 1 "Mr. John Smith, 66 years old, will succeed him as president")
(test-e2c 2 "Mr. John Smith, old, will succeed him as president")
(test-e2c 1 "A big black ugly dog chased me")
(test-e2c 1 "A big, black, ugly dog chased me")
(test-e2c 1 "The former astronaut was alone and afraid")
(test-e2c 2 "The alone astronaut was former")

;; ( PRINT "ADVERBS")

(test-e2c 1 "He is apparently an expert on dogs")
(test-e2c 2 "He knows apparently an expert on dogs")
(test-e2c 1 "Mary quickly walked out of the room")
(test-e2c 1 "Mary just walked out of the room")
(test-e2c 1 "Quickly, Mary walked out of the room")
(test-e2c 1 "Mary walked out of the room quickly")
(test-e2c 2 "Mary walked out of the room just")
(test-e2c 1 "He told them about the accident immediately")
(test-e2c 2 "He told them about the accident presumably")
(test-e2c 1 "He told them about the accident, presumably")
(test-e2c 1 "She is very careful about her work")
(test-e2c 1 "She works very carefully")
(test-e2c 2 "She very works carefully")
(test-e2c 1 "Is the piece easy enough for you")
(test-e2c 1 "Is the piece too easy for you")
(test-e2c 2 "Is the piece enough easy for you")
(test-e2c 1 "She is apparently an excellent pianist")
(test-e2c 2 "She married apparently an excellent pianist")
(test-e2c 1 "Only after the movie did he realize his mistake")
(test-e2c 2 "After the movie did he realize his mistake")
(test-e2c 1 "I may have taken cocaine a few times, but at no time did I inhale")
(test-e2c 2 "A few times may I have taken cocaine, but I inhaled at no time")
(test-e2c 1 "Never have I seen such a grotesque display of incompetence")
(test-e2c 2 "Often have I seen such a grotesque display of incompetence")
(test-e2c 1 "We like to eat at restaurants, particularly on weekends")
(test-e2c 1 "We like to eat at restaurants, usually on weekends")
(test-e2c 2 "We like to eat at restaurants, fortunately on weekends")
(test-e2c 1 "Such flowers are found mainly in Europe")
(test-e2c 2 "Such flowers are found apparently in Europe")
(test-e2c 2 "Such flowers are found mainly particularly in Europe")
(test-e2c 2 "Such flowers are found mainly")
(test-e2c 1 "Many people, particularly doctors, believe there is no health care crisis")
(test-e2c 2 "Many people, strongly doctors, believe there is no health care crisis")
(test-e2c 1 "I found a house that even John thinks we should buy")
(test-e2c 1 "He told me that even his mother likes me")
(test-e2c 2 "He told me that even, his mother likes me")
(test-e2c 1 "We put the pie straight in the oven")
(test-e2c 2 "We put the pie quickly in the oven")
(test-e2c 1 "We put the pie straight in")
(test-e2c 2 "We put the pie straight")
(test-e2c 1 "He lives high in the mountains")
(test-e2c 1 "He lives over by the lake")
(test-e2c 1 "He lives out down by the lake")
(test-e2c 2 "He lives out down by")
(test-e2c 1 "The apparently angry man walked out of the room")
(test-e2c 1 "The often underpaid administrators resent the invariably rude students and the understandably impatient professors")
(test-e2c 1 "The delicately lyrical tone of the cello contrasted with the fiercely percussive piano chords")
(test-e2c 1 "The always delicately lyrical tone was really beautiful")
(test-e2c 2 "The delicately always lyrical tone was really beautiful")
(test-e2c 2 "The delicately very lyrical tone was really beautiful")
(test-e2c 1 "Biochemically, I think the experiment has a lot of problems")
(test-e2c 1 "I think the experiment has a lot of problems biochemically")
(test-e2c 1 "It is biochemically an interesting experiment")
(test-e2c 1 "I'm not sure the results are biochemically valid")

;; ( PRINT "QUESTIONS AND RELATIVE CLAUSES")

(test-e2c 1 "Which dog did you chase")
(test-e2c 2 "Which dog you chased")
(test-e2c 1 "Which dog did you say you chased")
(test-e2c 2 "Which dog you said you chased")
(test-e2c 2 "Which dog did you say did you chase")
(test-e2c 1 "I wonder which dog he said you chased")
(test-e2c 2 "I wonder which dog did he say you chased")
(test-e2c 2 "I wonder which dog did he say did you chase")
(test-e2c 1 "What did John say he thought you should do")
(test-e2c 2 "What did John say did he think you should do")
(test-e2c 2 "What John said he thought you should do")
(test-e2c 1 "What Alice did really annoyed me")
(test-e2c 2 "Who Alice did really annoyed me")
(test-e2c 1 "Whoever designed this program didn't know what they were doing")
(test-e2c 2 "Who designed this program didn't know what they were doing")
(test-e2c 1 "Invite John and whoever else you want to invite")
(test-e2c 1 "The dog which Chris bought is really ugly")
(test-e2c 2 "The dog what Chris bought is really ugly")
(test-e2c 1 "I wonder whether we should go")
(test-e2c 2 "Whether should we go")
(test-e2c 1 "We can't decide whether to go to the party")
(test-e2c 2 "We can't decide who to go the party")
(test-e2c 2 "We can't decide whether to go the the party with")
(test-e2c 1 "I am wondering who to go to the party with")
(test-e2c 1 "I am wondering who to invite to the party")
(test-e2c 2 "I am wondering whether to invite to the party")
(test-e2c 2 "I am wondering the people to invite to the party")
(test-e2c 2 "Whether to go to the party")
(test-e2c 2 "Who to invite to the party")
(test-e2c 1 "Do you think we should go to the party")
(test-e2c 2 "What do you think we should go to the party")
(test-e2c 1 "How do you operate this machine")
(test-e2c 1 "How fast is the program")
(test-e2c 1 "How certain are you that John is coming")
(test-e2c 2 "How tired are you that John is coming")
(test-e2c 1 "How likely is it that he will come")
(test-e2c 2 "How likely is John that he will come")
(test-e2c 1 "How certain does he seem to be that John is coming")
(test-e2c 1 "How efficient a program is it")
(test-e2c 2 "Efficient a program is it")
(test-e2c 2 "How fast programs are they")
(test-e2c 2 "How fast the program is it")
(test-e2c 1 "How fast a program does he think it is")
(test-e2c 2 "How fast a program he thinks it is")
(test-e2c 2 "How fast programs does he think they are")
(test-e2c 2 "How big a dog chased you ")
(test-e2c 1 "I wonder how fast a program he thinks it is")
(test-e2c 2 "I wonder how fast a program does he think it is")
(test-e2c 1 "How much money did you earn")
(test-e2c 2 "How much money you earn")
(test-e2c 1 "I wonder how much money you earned")
(test-e2c 2 "I wonder how much money have you earned")
(test-e2c 1 "How much oil spilled")
(test-e2c 1 "How much do you swim")
(test-e2c 2 "How much you swim")
(test-e2c 1 "I wonder how much you swim")
(test-e2c 2 "I wonder how much do you swim")
(test-e2c 2 "I don't have how much money")
(test-e2c 1 "I don't have very much money")
(test-e2c 1 "I don't have much money")
(test-e2c 1 "How much did you read")
(test-e2c 2 "How much of the book you read")
(test-e2c 1 "How much of the book did you read")
(test-e2c 1 "I wonder how much of the book you read")
(test-e2c 1 "How many people died")
(test-e2c 1 "How many people did you see")
(test-e2c 2 "How many people you saw")
(test-e2c 1 "I wonder how many people you saw")
(test-e2c 1 "I wonder how many of the people you saw were students")
(test-e2c 1 "How did John do it")
(test-e2c 1 "I wonder how John did it")
(test-e2c 1 "How long will it last")
(test-e2c 2 "How big will it last")
(test-e2c 1 "How many years did it take to do it")
(test-e2c 1 "How big is the department")
(test-e2c 2 "How big the department is")
(test-e2c 2 "I wonder how big is the department")
(test-e2c 1 "I wonder how big the department is")
(test-e2c 2 "I wonder how big departments they are")
(test-e2c 2 "I wonder how a department it is")
(test-e2c 1 "I wonder how big a department it is")
(test-e2c 1 "How important is it to turn the computer off")
(test-e2c 1 "I wonder how important it is to turn off the computer")
(test-e2c 2 "I wonder how important is it to turn off the computer")
(test-e2c 1 "How quickly did Joe run")
(test-e2c 2 "How quickly Joe ran")
(test-e2c 1 "I know how quickly you ran")
(test-e2c 2 "I know how quickly did you run")
(test-e2c 2 "He ran I know how quickly")
(test-e2c 2 "Quickly did Joe run")
(test-e2c 2 "Very quickly did Joe run")
(test-e2c 2 "I know very quickly did Joe run")
(test-e2c 2 "I know quickly did John run")
(test-e2c 1 "How much more quickly did you run")
(test-e2c 2 "How much more quickly you run")
(test-e2c 2 "I wonder how much more quickly did he run")
(test-e2c 1 "I wonder how much more quickly he ran")
(test-e2c 1 "How much more quickly did he run than Joe")
(test-e2c 1 "How much more should we work on this")
(test-e2c 1 "How much further do you think we should drive tonight")
(test-e2c 1 "I don't know how much longer I can tolerate this")
(test-e2c 1 "How much bigger is the dog")
(test-e2c 2 "How much bigger dogs are they")
(test-e2c 2 "How much bigger dogs ran")
(test-e2c 2 "How big dogs run")
(test-e2c 1 "How much further did you run")
(test-e2c 1 "How much more oil spilled")
(test-e2c 1 "How much more spilled")
(test-e2c 1 "How much more oil did they spill")
(test-e2c 1 "How much more did they spill")
(test-e2c 2 "How much more they spilled")
(test-e2c 1 "I wonder how much oil spilled")
(test-e2c 1 "I wonder how much oil they spilled")
(test-e2c 2 "How much more efficient programs are available")
(test-e2c 1 "How many dogs ran")
(test-e2c 1 "How many ran")
(test-e2c 1 "How many dogs did you see")
(test-e2c 1 "How many more people did you see")
(test-e2c 1 "How many more people do you think will come")
(test-e2c 1 "I wonder how many more people he thinks will come")
(test-e2c 2 "I wonder how many more people does he think will come")
(test-e2c 1 "How many times did you do it")
(test-e2c 2 "How many times you did it")
(test-e2c 1 "I wonder how many times you did it")
(test-e2c 2 "How many more stupid times did you do it")
(test-e2c 1 "How many years ago did you do it")
(test-e2c 2 "Many years ago did you do it")
(test-e2c 2 "How many years did you do it")
(test-e2c 1 "I wonder how many years ago you did it")
(test-e2c 2 "How many years ago you did it")
(test-e2c 1 "I'll show you the house where I met your mother")
(test-e2c 2 "I'll show you the house which I met your mother")
(test-e2c 1 "This is the man whose dog I bought")
(test-e2c 2 "This is the man which dog I bought")
(test-e2c 1 "I wonder where John is")
(test-e2c 2 "I wonder where John hit")
(test-e2c 1 "The dogs, some of which were very large, ran after the man")
(test-e2c 1 "The dogs, some of which I had seen before, ran after the man")
(test-e2c 2 "The dogs some of which were very large ran after the man")
(test-e2c 1 "The box contained many books, some of which were badly damaged")
(test-e2c 2 "Some of which were badly damaged")
(test-e2c 2 "The box contained many books, some were badly damaged")
(test-e2c 2 "The box contained many books, some of the books were badly damaged")
(test-e2c 1 "The dogs, some of them very large, ran after the man")
(test-e2c 2 "The dogs, some of the dogs very large, ran after the man")
(test-e2c 2 "Some of them very large ran after the man")
(test-e2c 1 "The man was chased by dogs, some of them very large")
(test-e2c 1 "I believe it was John who stole the priceless documents")
(test-e2c 2 "I believe Fred was John who stole the priceless documents")
(test-e2c 1 "It seems to have been Einstein who first came up with the idea")
(test-e2c 2 "There seems to have been Einstein who first came up with the idea")
(test-e2c 2 "It hopes to have been Einstein who first came up with the idea")
(test-e2c 2 "The book discussed Einstein who first came up with the idea")
(test-e2c 2 "Stravinsky was in Paris that Debussy first heard Balinese music")
(test-e2c 1 "It was in Paris that Debussy first heard Balinese music")
(test-e2c 1 "It must have been there that he realized his destiny")
(test-e2c 2 "It tried to have been there that he realized his destiny")
(test-e2c 2 "He composed some good music that he realized his destiny")
(test-e2c 2 "It was quickly that he wrote his first symphony")
(test-e2c 1 "Wasn't it in 1955 that Sally first met Joe")
(test-e2c 1 "Whatever the outcome, I'm sure he'll still be popular")
(test-e2c 2 "Whatever an outcome, I'm sure he'll still be popular")
(test-e2c 1 "Whatever the outcome is, I'm sure he'll still be popular")
(test-e2c 2 "What the outcome is, I'm sure he'll still be popular")

;; ( PRINT "IT-THERE-THIS")

(test-e2c 1 "There is a dog in the park")
(test-e2c 2 "There is chasing dogs")
(test-e2c 2 "There are a dog in the park")
(test-e2c 1 "Does there seem to be a dog in the park?")
(test-e2c 2 "Does there want to be a dog in the park?")
(test-e2c 1 "There seems to appear to have been likely to be a problem")
(test-e2c 2 "There seems to appear to have been likely to be problems")
(test-e2c 2 "There seems to appear to have been likely to be stupid")
(test-e2c 1 "There was an attempt to kill Rod")
(test-e2c 1 "The man there was an attempt to kill died")
(test-e2c 1 "There was a problem, but we solved it")
(test-e2c 1 "It is likely that Rod died")
(test-e2c 2 "Joe is likely that Rod died")
(test-e2c 1 "It is clear who killed Rod")
(test-e2c 2 "Joe is clear who killed Rod")
(test-e2c 1 "It may not be possible to fix the problem")
(test-e2c 1 "Grace may not be possible to fix the problem")
(test-e2c 1 "It is important that women be ready when they make these choices")
(test-e2c 2 "It is clear that women be ready when they make these choices")
(test-e2c 2 "Joe is important that women be ready when they make these choices")
(test-e2c 1 "flowers are red to attract bees")
(test-e2c 1 "I made it clear that I was angry")
(test-e2c 2 "I made Anne clear that I was angry")
(test-e2c 1 "Dick is easy to hit")
(test-e2c 2 "Dick is black to hit")
(test-e2c 1 "It is important to fix the problem")
(test-e2c 1 "Dick is important to fix the problem")
(test-e2c 1 "The man it is likely that John hit died")
(test-e2c 2 "The man Joe is likely that Dick hit died")
(test-e2c 1 "Does it seem likely that Ann will come")
(test-e2c 1 "Does Ann act glad that Joe came")
(test-e2c 2 "Does it act likely that Joe came")
(test-e2c 1 "It doesn't matter what Ted does")
(test-e2c 2 "Joe doesn't matter what Ted does")
(test-e2c 1 "I want it to be possible to use the program")
(test-e2c 1 "I want Joe to be possible to use the program")
(test-e2c 1 "I want it to be clear that it was my idea")
(test-e2c 2 "I asked it to be clear that it was my idea")
(test-e2c 1 "I want it to be obvious how to use the program")
(test-e2c 2 "I want Emily to be obvious how to use the program")
(test-e2c 1 "I want Joe to be easy to hit")
(test-e2c 1 "It is likely they will come")
(test-e2c 2 "Joe is likely they will come")
(test-e2c 1 "This is because he is extremely famous")
(test-e2c 1 "The trial is because he is extremely famous")
(test-e2c 1 "The excitement over the trial is because he is extremely famous")
(test-e2c 1 "This seems to have been because he is extremely famous")

;; ( PRINT "COMPARATIVES")

(test-e2c 1 "Our program works more elegantly than yours")
(test-e2c 1 "Ours works more elegantly than yours does")
(test-e2c 1 "Ours works more elegantly than yours works")
(test-e2c 2 "Ours works more elegant than yours")
(test-e2c 2 "Ours is more elegant than yours works")
(test-e2c 1 "Our program works more elegantly than efficiently")
(test-e2c 1 "Our program is more elegant than efficient")
(test-e2c 1 "Our program works better than yours")
(test-e2c 1 "We do this more for pleasure than for money")
(test-e2c 1 "He is more likely to go than to stay")
(test-e2c 2 "He is more likely than to stay")
(test-e2c 2 "He is more black to go than to stay")
(test-e2c 1 "He is more likely to go than he is to stay")
(test-e2c 1 "He is more likely to go than John is")
(test-e2c 1 "It is more likely that Joe died than that Fred died")
(test-e2c 1 "It is more likely that Joe died than it is that Fred died")
(test-e2c 2 "John is more likely that Joe died than it is that Fred died")
(test-e2c 2 "It is more likely that Joe died than John is that Fred died")
(test-e2c 1 "It is easier to ignore the problem than to solve it")
(test-e2c 1 "It is easier to ignore the problem than it is to solve it")
(test-e2c 2 "Greg is easier to ignore the problem than to solve it")
(test-e2c 1 "Our program is easier to use than to understand")
(test-e2c 2 "Our program is easier to use it than to understand")
(test-e2c 1 "I am more happy now than I was in college")
(test-e2c 2 "I am more happy now than I earned in college")
(test-e2c 1 "He is more a teacher than a scholar")
(test-e2c 1 "I make more money in a month than John makes in a year")
(test-e2c 1 "I make more money in a month than John dies in a year")
(test-e2c 1 "I hit more the dog than the cat")
(test-e2c 1 "I have more money than John has time")
(test-e2c 1 "I have more dogs than John has five cats")
(test-e2c 1 "I have more money than John has a dog")
(test-e2c 1 "She interviewed more programmers than were hired")
(test-e2c 2 "She interviewed more programmers than was hired")
(test-e2c 1 "I am as intelligent as John")
(test-e2c 1 "I earn as much money as John does")
(test-e2c 1 "I am as intelligent as John does")
(test-e2c 1 "I earn as much money in a month as John earns in a year")
(test-e2c 2 "I earn as much money in a month than John earns in a year")
(test-e2c 1 "Our program was better than had been expected")
(test-e2c 2 "Our program was better than had been argued")
(test-e2c 2 "Our program was better than had been responded")
(test-e2c 1 "Our program was better than was expected")
(test-e2c 2 "Our program was better than were expected")
(test-e2c 1 "More people came to the party than were expected")
(test-e2c 1 "More people came to the party than was expected")
(test-e2c 1 "Our program did not run as quickly as expected")
(test-e2c 2 "Our program did not run as quickly as said")
(test-e2c 1 "How much faster is our program than theirs")
(test-e2c 2 "How much faster our program is than theirs")
(test-e2c 1 "The more quickly we write the program, the more money we will earn")
(test-e2c 2 "The more people like the program")
(test-e2c 2 "The people like the program, the more money we will earn")
(test-e2c 1 "The better the program is, the more people will like it")
(test-e2c 1 "The better the program, the more people will like it")
(test-e2c 2 "The better a program, the more people will like it")
(test-e2c 1 "The less likely it is that we can parse this, the easier it is to understand ")

;; ( PRINT " \"SO THAT\", \"SUCH...THAT\"")

(test-e2c 1 "The shuttle is so big that it has to be carried on the back of a jet ")
(test-e2c 2 "The shuttle is big that it has to be carried on the back of a jet ")
(test-e2c 1 "So many people attended that they spilled over into several neighboring fields")
(test-e2c 2 "Many people attended that they spilled over into several neighboring fields")
(test-e2c 1 "The program has so many problems that you should probably just rewrite it")
(test-e2c 2 "The program has many problems that you should probably just rewrite it")
(test-e2c 1 "I love her so much that I can't let her go")
(test-e2c 2 "I love her very much that I can't let her go")
(test-e2c 1 "He ran home so quickly that his mother could hardly believe he had called from school")
(test-e2c 2 "He ran home quickly that his mother could hardly believe he had called from school")
(test-e2c 1 "She presented her case with such eloquence that we could only admire her")
(test-e2c 2 "She presented her case with eloquence that we could only admire her")

;; ( PRINT "SUBORDINATING CONJUNCTIONS")

(test-e2c 1 "The man we saw when we went to Paris is here")
(test-e2c 2 "The man we saw but we went to Paris is here")
(test-e2c 1 "You should see a play while in London")
(test-e2c 2 "You should see a play after in London")
(test-e2c 1 "I left the party after seeing Ann there")
(test-e2c 2 "I left the party because seeing Ann there")
(test-e2c 2 "I left the party despite I saw Ann there")
(test-e2c 1 "Because I didn't see Ann, I left")
(test-e2c 2 "Therefore I didn't see Ann, I left")
(test-e2c 1 "I left, therefore I didn't see Ann")
(test-e2c 1 "But I really wanted to see her")
(test-e2c 2 "After I really wanted to see her")
(test-e2c 1 "As I suspected, he had already left")
(test-e2c 2 "Because I suspected, he had already left")
(test-e2c 2 "I suspected, he had already left")
(test-e2c 2 "I suspected")
(test-e2c 1 "Some grammars are better than others, as we have proved")
(test-e2c 1 "As had been expected, the party was a big success")
(test-e2c 2 "As had been green, the party was a big success")
(test-e2c 2 "As had wanted to be expected, the party was a big success")
(test-e2c 2 "As had expected the party to be a success, it was a success")
(test-e2c 1 "In the event that the case goes to trial, I'm sure Clinton will be acquitted")
(test-e2c 2 "In the scandal that the case goes to trial, I'm sure Clinton will be acquitted")
(test-e2c 1 "He should be acquitted, on the grounds that there's no evidence")
(test-e2c 1 "The public seem to love him, no matter what he does")
(test-e2c 2 "The public seem to love him, no matter the stupid things he does")

;; ( PRINT "COORDINATING CONJUNCTIONS")

(test-e2c 1 "I went to the store and got a gallon of milk")
(test-e2c 2 "I got and went a gallon of milk")
(test-e2c 1 "I got a gallon of milk and some eggs")
(test-e2c 1 "I went to the store, got a gallon of milk, and returned the eggs")
(test-e2c 2 "I went to the store, got a gallon of milk, and some eggs")
(test-e2c 1 "Mary, Joe and Louise are coming to the party")
(test-e2c 1 "Neither Mary nor Louise are coming to the party")
(test-e2c 1 "I am ready and eager to go to the party")
(test-e2c 1 "She handled it skillfully and with compassion")
(test-e2c 1 "I told him that I hated him and that I never wanted to see him again")
(test-e2c 1 "He told me why he was here and what he was doing")
(test-e2c 2 "He told me why he was here and that he hated me")
(test-e2c 1 "Although he likes me and he respects me, he says he needs some privacy")
(test-e2c 1 "Your house and garden are very attractive")
(test-e2c 1 "I am in New York and I would like to see you")
(test-e2c 1 "This is not the man we know and love")
(test-e2c 2 "This is not the man we know and love him")
(test-e2c 1 "The coverage on TV and on the radio has been terrible")
(test-e2c 2 "The coverage on TV and I have seen has been terrible")
(test-e2c 1 "The sky is blue, so it is likely that Joe will come")
(test-e2c 2 "It is blue and likely that Joe will come")
(test-e2c 1 "That is the man for whom and with whom Joe works")
(test-e2c 2 "That is the man for whom and with Janet Joe works")
(test-e2c 2 "When did Joe and John did leave the party")
(test-e2c 1 "My dog, cat, and cousin's friend came")
(test-e2c 2 "My dog, cat, horse, mouse, and his cow left")
(test-e2c 1 "My dog, cat, horse, and mouse, and his cow left")
(test-e2c 1 "you should not only ask for your money back, but demand it")
(test-e2c 1 "I was both angry and sad at the same time")

;; ( PRINT "PLURALIZATION IN CONJUNCTIONS")

(test-e2c 1 "There is neither a dog nor a cat here")
(test-e2c 2 "There are neither a dog nor a cat here")
(test-e2c 1 "There is a dog or a cat here")
(test-e2c 2 "There are a dog or a cat here")
(test-e2c 2 "There are a dog and a cat here")
(test-e2c 1 "There is a dog and a cat here")
(test-e2c 1 "He and I are friends")
(test-e2c 1 "neither I nor my friend knows what happened")
(test-e2c 1 "neither I nor my friend know what happened")
(test-e2c 1 "Either I or my friend knows what happened")
(test-e2c 1 "Either I or my friend know what happened")
(test-e2c 1 "The dog and cats know what happened")
(test-e2c 2 "The dog and cats knows what happened")
(test-e2c 1 "Are a dog and a cat here")
(test-e2c 2 "Is a dog and a cat here")
(test-e2c 2 "Is John and I invited")
(test-e2c 1 "Are John and I invited")
(test-e2c 1 "Is John or I invited")
(test-e2c 1 "Are John or I invited")
(test-e2c 1 "Is neither John nor I invited")
(test-e2c 1 "Are neither John nor I invited")

;; ( PRINT "GERUNDS")

(test-e2c 1 "Playing the piano bothers John")
(test-e2c 1 "Releasing the program at this point would annoy our competitors")
(test-e2c 1 "The playing of the piano really bothers John")
(test-e2c 2 "The playing the piano really bothers John")
(test-e2c 1 "Telling Joe about the party would create a real problem")
(test-e2c 2 "The telling Joe about the party could create a real problem")
(test-e2c 1 "Your telling Joe about the party could create a real problem")
(test-e2c 1 "Telling Joe that Sue was coming to the party would create a real problem")
(test-e2c 1 "Telling would create a real problem")
(test-e2c 1 "I want her to know about it, but the telling won't be easy")
(test-e2c 2 "The telling her won't be easy")
(test-e2c 2 "Some children like to tease")
(test-e2c 1 "Teasing can be very cruel")
(test-e2c 1 "Your telling John to leave may have destroyed your relationship")
(test-e2c 1 "The graduating of Fred changes the situation")
(test-e2c 1 "The sleeping of students is becoming a big problem")
(test-e2c 1 "The sleeping of students can ruin a lecture")
(test-e2c 1 "Buying of shares was brisk on Wall Street today")
(test-e2c 1 "The sleeping in class is becoming a big problem")
(test-e2c 2 "The telling John to leave was stupid")
(test-e2c 2 "The inviting your mother was stupid")
(test-e2c 2 "The showing how to use the program seemed to interest people")
(test-e2c 2 "The attempting to go to the party angered Joe")
(test-e2c 1 "The showing of the program seemed to impress people")
(test-e2c 1 "The sleeping of students described by Fred is a big problem")
(test-e2c 1 "The sleeping of students I told you about is a big problem")
(test-e2c 1 "The frequent sleeping of students is a big problem")
(test-e2c 1 "His hitting of the dog didn't help matters")
(test-e2c 1 "Some hitting of dogs will solve the problem")
(test-e2c 1 "the drug running here has become a massive problem")
(test-e2c 1 "He made a mistake in inviting John")
(test-e2c 1 "He made a mistake in the inviting of John")
(test-e2c 1 "I should have talked to you before inviting John")
(test-e2c 1 "I should have talked to you before the inviting of John")

;; ( PRINT "SPECIAL SUBJECTS: INFINITIVES, CLAUSES, AND INDIRECT QUESTIONS")

(test-e2c 1 "To pretend that our program is usable in its current form would be silly")
(test-e2c 2 "To pretend that our program is usable in its current form would be happy")
(test-e2c 1 "That our program will be immediately accepted is hardly likely")
(test-e2c 2 "That our program will be immediately accepted wrote the program")
(test-e2c 2 "Is that our program will be accepted likely")
(test-e2c 2 "That our program will be accepted seems likely that our program will be accepted")
(test-e2c 1 "Whether we should go to the party is the important question")
(test-e2c 2 "Whether we should go to the party annoys me")

;; ( PRINT "LONGER CONSTRUCTIONS, PUNCTUATION, AND CAPITALIZATION")

(test-e2c 1 "Using the conventional Minuet form, Beethoven produced a piece of great originality")
(test-e2c 1 "Written in 1820, the symphony shows a new level of maturity for the composer")
(test-e2c 1 "Abandoned by his friends, he left Vienna three years later")
(test-e2c 1 "In Vienna, Beethoven met someone who would later be greatly influenced by him: Franz Schubert")
(test-e2c 2 "In Vienna, Beethoven met someone who would later be greatly influenced by him; Franz Schubert")
(test-e2c 1 "Today I did something very important: I bought a dog")
(test-e2c 2 "The store where I did something very important: I bought a dog was closed today")
(test-e2c 1 "It has been said that Schubert ran out of the room when he met Beethoven; but we now know this is untrue")
(test-e2c 1 "An important question remains: did Beethoven know about Schubert's music")
(test-e2c 1 "She just wanted one thing: to be a professional skater")
(test-e2c 1 "She knew one thing: that she would be a professional skater")
(test-e2c 1 "I agree that, in some ways, your program is better")
(test-e2c 1 "I agree that in some ways, your program is better")
(test-e2c 2 "I agree that, in some ways your program is better")
(test-e2c 1 "That is the man who, in Joe's opinion, we should hire")
(test-e2c 2 "That is the man, in Joe's opinion, we should hire")
(test-e2c 2 "That is the man who, in Joe's opinion we should hire")
(test-e2c 1 "I know you hate Bill, but why did you send him that nasty note")
(test-e2c 2 "I know you hate Bill, because why did you send him that nasty note")
(test-e2c 1 "But why did you send him that nasty note")
(test-e2c 1 "If John was with Lisa last night, who went to the movie with Diane")
(test-e2c 2 "Although John was with Lisa last night, who went to the movie with Diane")
(test-e2c 1 "We need a President who understands us")
(test-e2c 1 "We need a president who understands us")
(test-e2c 2 "We need a Melvin who understand us")
(test-e2c 1 "The Zongle of Bongle Dongle resigned today")
(test-e2c 2 "A Zongle with a Bongle Dongle resigned today")
(test-e2c 1 "The National Association of Linguists is meeting here")
(test-e2c 2 "An Association that many Linguists belong to is meeting here")
(test-e2c 1 "An association that many linguists belong to is meeting here")
(test-e2c 1 "If you were a middle-class American without a job, who would you vote for")
(test-e2c 1 "Many Croats who had fled their homes are now returning to them")
(test-e2c 2 "Many Croat who had fled their homes are now returning to them")
(test-e2c 1 "Chinese is a wonderful language, Chinese food is nice, and the Chinese are nice people")
(test-e2c 2 "Armenian is a wonderful language, Armenian food is great, and the Armenian are nice people")
(test-e2c 1 "Armenian is a wonderful language, Armenian food is great, and the Armenians are nice people")
(test-e2c 1 "Danish is a wonderful language, Danish food is great, and the Danish are nice people")
(test-e2c 1 "The Danes are nice people")
(test-e2c 1 "Dr Jane Smith lives on Main St")
(test-e2c 1 "Dr. Jane Smith lives on Main St")
(test-e2c 2 "Dr. Jane. Smith. lives on Main. St")
(test-e2c 1 "Dr. J.G.D. Smith lives on Main St")
(test-e2c 2 "A Dr. lives on this St")
(test-e2c 1 "Mr. Smith (a lawyer for Kodak) refused to comment")
(test-e2c 1 "Mr. Smith -- a lawyer for Kodak -- refused to comment")
(test-e2c 1 "We left (carrying the dog) and Fred followed")
(test-e2c 1 "I have $50, but I want a $50000 car")
(test-e2c 1 "10% of the employees here do 90% of the work")
(test-e2c 1 "Zangbert stock fell 30% to $2.50 yesterday, but jumped 10% today in heavy trading")
(test-e2c 2 "Zangbert stock fell %, but jumped to $ today")
(test-e2c 1 "With a 5% raise, I can get a $50000 car")
(test-e2c 1 "\"What are you doing?\" she asked")
(test-e2c 1 "\"This is what I'm going to do,\" he replied")
(test-e2c 1 "\"This is what I'm going to do!\" he replied")
(test-e2c 2 "\"This is what I'm going to do.\" he replied")
(test-e2c 2 "\"This is what I'm doing to do\" he replied")
(test-e2c 1 "\"On second thought,\" he said, \"this is what I'm going to do\"")
(test-e2c 1 "\"Quotation marks\" are simply \"ignored\" by our \"program\"")
(test-e2c 1 "Sometimes, people do this: They follow the colon with a capital letter")
(test-e2c 2 "However, they never do this; They don't follow a semi-colon with a capital letter")
(test-e2c 1 "John said: \"This is another use of colons one sometimes sees\"")
(test-e2c 1 "Formerly, he had worked for Brody, McGill & Demson")
(test-e2c 2 "He was unhappy & underpaid ")
(test-e2c 1 "The rally, at 6:00 last night, was attended by 1.1 million people")

;; ( PRINT "DONE")



;;% The various grammar qualifying tests are given below. 
;;%
;;% Tests t1,t2,... should parse successfully. 
;;% Tests f1,f2,... should show parse failures.
;;% Tests q1,q2,... should be successful querys. 
;;% Tests r1,r2,... should be unsuccessful querys. 

runhistests:-(test-e2c t1 "the men cry") 
(test-e2c t2 "the man cries") 
(test-e2c t3 "the man saw the dogs") 
(test-e2c t4 "he wants the dog") 
(test-e2c t5 "he wants to be happy") 
(test-e2c t6 "he wants the man to see the dog") 
(test-e2c t7 "he is happy to be a dog") 
(test-e2c t8 "i saw the happy red saw") 
(test-e2c t9 "he saw the man saw the wood with the saw") 
(test-e2c t10  "jack goes to see the fish") 
(test-e2c t11  "i will hide my hat in the drawer") 
(test-e2c t12  "i hid my hat in the drawer") 
(test-e2c t13  "i was hiding my hat in the drawer") 
(test-e2c t14  "my hat will be hidden in the drawer") 
(test-e2c t15  "my hat was hidden in the drawer") 
(test-e2c t16  "my hat was being hidden in the drawer") 
(test-e2c t17  "my hat is hidden in the drawer") 
(test-e2c t18  "i can see the house") 
(test-e2c t19  "i will have seen the house") 
(test-e2c t20  "i was watching the movie") 
(test-e2c t21  "i should have been watching the movie") 
(test-e2c t22  "he was sad to see the man cry to the fish") 
(test-e2c t23  "jack has to see a doctor") 
(test-e2c t24  "the cat had to be found") 
(test-e2c t25  "john has to be winning the race") 
(test-e2c t26  "the book would have had to have been found by jack") 
(test-e2c t27  "i love you") 
(test-e2c t28  "you love me") 
(test-e2c t29  "you love yourselves") 
(test-e2c t30  "i think that you loved me") 
(test-e2c t31  "jack thought of jill") 
(test-e2c t32  "it seems that jack is a man") 
(test-e2c t33  "jack was seen by jill") 
(test-e2c t34  "jack was seen") 
(test-e2c t35  "jack can see the dog") 
(test-e2c t36  "jill saw that jack was a man") 
(test-e2c t37  "jack bought a ticket") 
(test-e2c t38  "a ticket was bought by jill") 
(test-e2c t39  "jill gave jack a ticket") 
(test-e2c t40  "jill gave a ticket to jack") 
(test-e2c t41  "jill thinks that jack stole the book") 
(test-e2c t42  "jack will be seen going to the house") 
(test-e2c t43  "jack was seen laughing") 
(test-e2c t44  "the man in the corner was crying") 
(test-e2c t45  "jill was crying in the corner") 
(test-e2c t46  "george said that henry left in the car") 
(test-e2c t47  "i wanted the dog in the house") 
(test-e2c t48  "i kept the dog in the house") 
(test-e2c t49  "i put the dog in the house") 
(test-e2c t50  "we painted all the walls with cracks") 
(test-e2c t51  "the horse raced past the barn fell") 
(test-e2c t52  "the horse that was raced past the barn fell") 
(test-e2c t53  "have the students who missed the exam take it today") 
(test-e2c t54  "have the students who missed the exam taken it today") 
(test-e2c t55  "i saw the man in the house with a telescope") 
(test-e2c t56  "he wants to go home quickly") 
(test-e2c t57  "he wants to go quickly home") 
(test-e2c t58  "he wants to quickly go home") 
(test-e2c t59  "he wants quickly to go home") 
(test-e2c t60  "he quickly wants to go home") 
(test-e2c t61  "jack was pleased with the prize") 
(test-e2c t62  "who are they") 
(test-e2c t63  "who ate the pizza") 
(test-e2c t64  "what did you put the book in") 
(test-e2c t65  "what book did he steal") 
(test-e2c t66  "where did you put the book") 
(test-e2c t67  "how quickly did he run") 
(test-e2c t68  "whose book did you find") 
(test-e2c t69  "in what store did you buy the picture") 
(test-e2c t70  "i am the man who you saw at the store") 
(test-e2c t71  "i destroyed the exam in which you found the error") 
(test-e2c t72  "i am the man whose book you stole") 
(test-e2c t73  "this is the paper jack read") 
(test-e2c t74  "this is the damage caused by the storm") 
(test-e2c t75  "this is the issue creating the argument") 
(test-e2c t76  "the man who we saw cried") 
(test-e2c t77  "this is the mouse that the cat ate") 
(test-e2c t78  "the paper jack read is here") 
(test-e2c t79  "the man gave bill the money") 
(test-e2c t80  "george gave the rug to ernest") 
(test-e2c t81  "george gave the money to the man") 
(test-e2c t82  "bill was given the money by the man") 
(test-e2c t83  "the rug was given to ernest by george") 
(test-e2c t84  "the money was given to the man by george") 
(test-e2c t85  "the man bill was given the money by is here") 
(test-e2c t86  "the rug that was given to ernest by george is red") 
(test-e2c t87  "the man whom the money was given to by george is rich") 
(test-e2c t88  "where are you") 
(test-e2c t89  "where am i") 
(test-e2c t90  "there were many people at the game") 
(test-e2c t91  "few cars are fast") 
(test-e2c t92  "a few cars are fast") 
(test-e2c t93  "few of the cars are fast") 
(test-e2c t94  "very few of the cars are fast") 
(test-e2c t95  "a few of the cars are fast") 
(test-e2c t96  "all cars are fast") 
(test-e2c t97  "all the cars are fast") 
(test-e2c t98  "all of the cars are fast") 
(test-e2c t99  "who should we invite to the party") 
(test-e2c t100  "water is necessary for life") 
(test-e2c t101  "a bunch of dogs are in the yard") 
(test-e2c t102  "a bunch of dogs is in the yard") 
(test-e2c t103  "bunches of dogs are in the yard") 
(test-e2c f1-bad "the men cries") 
(test-e2c f2-bad "the man cry") 
(test-e2c f3-bad "the man saw to be happy") 
(test-e2c f4-bad "he wants") 
(test-e2c f5-bad "he wants the man saw the dog") 
(test-e2c f6-bad "he see the man") 
(test-e2c f7-bad "i have should been watching the movie") 
(test-e2c f8-bad "i go caught fish") 
(test-e2c f9-bad "you was hiding in the drawer") 
(test-e2c f10-bad "jack can hidden") 
(test-e2c f11-bad "jack has to have to see a doctor") 
(test-e2c f12-bad "john had to played the violin") 
(test-e2c f13-bad "jack would to go to the movies") 
(test-e2c f14-bad "eat john the pizza") 
(test-e2c f15-bad "me love you") 
(test-e2c f16-bad "you love you") 
(test-e2c f17-bad "yourself loves me") 
(test-e2c f18-bad "he has might see the movie already") 
(test-e2c f19-bad "jack was seen flown to his house") 
(test-e2c f20-bad "we saw that was man") 
(test-e2c f21-bad "jack was seen go") 
(test-e2c f22-bad "my hat was being hidden the dog in the drawer") 
(test-e2c f23-bad "the man in corner had a violin") 
(test-e2c f24-bad "he wants to go home happy") 
(test-e2c f25-bad "he wants to happy go home") 
(test-e2c f26-bad "he happy wants to go home") 
(test-e2c f27-bad "who are i") 
(test-e2c f28-bad "it is house") 
(test-e2c f29-bad "where am you") 
(test-e2c f30-bad "where are i") 
(test-e2c f31-bad "few car are fast") 
(test-e2c f32-bad "few cars is fast") 
(test-e2c f33-bad "a few car are fast") 
(test-e2c f34-bad "all a cars are fast") 
(test-e2c f35-bad "many all cars are fast") 
(test-e2c f36-bad "waters is necessary for life") 
(test-e2c f37-bad "bunches of dogs is in the yard") 
(test-e2c f38-bad "bunch of dogs are in the yard") 
(test-e2c f39-bad "bunch of dogs is in the yard") 
(test-e2c q1 "did you see the car") 
(test-e2c q2 "can i catch it") 
(test-e2c q3 "did i eat the pizza") 
(test-e2c q4 "did the boy climb in the window") 
(test-e2c q5 "do i have a pen") 
(test-e2c q6 "did you see the cat") 
(test-e2c q7 "can i try it") 
(test-e2c q8 "is the boy hiding") 
(test-e2c q9 "is the boy hidden") 
(test-e2c q10  "is the boy at home") 
(test-e2c q11  "are the boys all men") 
(test-e2c q12  "is it necessary for jack to go to the store") 
(test-e2c q13  "was jack a man") 
(test-e2c q14  "are the boys in the house") 
(test-e2c q15  "are the boys playing") 
(test-e2c q16  "are the boys playing with the house") 
(test-e2c q17  "where am i going") 
(test-e2c q18  "what are you doing") 
(test-e2c q19  "what do you do") 
(test-e2c q20  "why do you want to do that") 
(test-e2c q21  "were there many people at the game") 
(test-e2c q22  "what did you put in the cupboard") 
(test-e2c q23  "why did you put the bottle in the cupboard") 
(test-e2c r1-bad "can the boys in the house") 
(test-e2c r2-bad "where do i going") 
(test-e2c r3-bad "what am you doing") 
(test-e2c r3-bad "what are you do") 
(test-e2c r4-bad "what did you put the bottle in the cupboard").

;;% "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape"
;; runMudTest:-
(test-e2c 'NpcCol1004-Troi712 "Counselor Deanna Troi is here")
(test-e2c 'Area1008 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'NpcCol1002-Worf720 "Lieutenant Worf is here, looking pretty mean")
(test-e2c 'NpcCol1001-Data672 "Lieutenant Commander Data is here, trying to be more human")
(test-e2c 'Area1028 "Two large windows offer a great view of space")
(test-e2c 'Area1012 "It's quite a large room, with a very high ceiling and a lot of floor space")
(test-e2c 'Area1032 "Several vertical rows of lights make this place very well lit")
(test-e2c 'Area1026 "A thick stack of mats have been piled high in one corner, which can be used for different activities")
(test-e2c 'Area1033 "From here, you can access the other decks on the Enterprise")
(test-e2c 'Area1034 "You're in the turbolift")
(test-e2c 'Area1023 "Several different paintings have been hung from the walls, and a small couch and a recliner are positioned around a coffee table")
(test-e2c 'Area1027 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1039 "You feel very cold")
(test-e2c 'Area1004 "A computer control panel is situated in the northwestern corner of the room, which is where the force fields for the cells are controlled")
(test-e2c 'Area1020 "Three long tables surrounded by chairs stretch across the room")
(test-e2c 'Area1037 "A large glass rectangular table sits in the middle of the room, surrounded by about a dozen comfortable looking office chairs")
(test-e2c 'Area1015 "A large glass window in the northern part of the room separates the doctor's office with the rest of the room")
(test-e2c 'Area1010 "You're in the turbolift")
(test-e2c 'Area1008 "You see the holodeck's control panel beside the holodeck door, and it has some information on it")
(test-e2c 'Area1040 "You're floating in outer space right above the USS Enterprise")
(test-e2c 'NpcCol1006-Picard701 "Captain Jean-Luc Picard is standing here, watching you")
(test-e2c 'Area1011 "You're in the turbolift")
(test-e2c 'Area1012 "You can see several hundred plastic crates and barrels with the Starfleet insignia on them stacked right up to the ceiling")
(test-e2c 'Area1026 "Captain Picard likes to come here to practice his fencing")
(test-e2c 'Area1034 "Several vertical rows of lights make this place very well lit")
(test-e2c 'Area1002 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1003 "You're in the middle of Data's quarters")
(test-e2c 'Area1014 "You've arrived in Riker's quarters")
(test-e2c 'Area1023 "A neatly made bed is partially hidden behind a curtain at the northern part of the room")
(test-e2c 'Area1027 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1039 "A large grey door leads into the Enterprise's Shuttle Bay")
(test-e2c 'Area1031 "You can see several hundred plastic crates and barrels with the Starfleet insignia on them stacked right up to the ceiling")
(test-e2c 'NpcCol1012-Ensign748 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1018 "You're in Doctor Crusher's quarters")
(test-e2c 'Area1010 "The turbolift walls have been rounded off, making it in the shape of a tube")
(test-e2c 'Area1006 "You're in the Enterprise transporter room")
(test-e2c 'NpcCol1012-Ensign732 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1039 "You're floating in outer space right beside the USS Enterprise")
(test-e2c 'Area1011 "The turbolift walls have been rounded off, making it in the shape of a tube")
(test-e2c 'Area1028 "A small partition at the northern part of the room contains Picard's sleeping area")
(test-e2c 'Area1037 "This is where the senior officers of the Enterprise meet and discuss important issues")
(test-e2c 'Area1038 "You find yourself on the lower half of the main bridge of the USS Enterprise")
(test-e2c 'Area1002 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1003 "Some easils and paintings have been left scattered around the southern part of the room, while a huge computer screen showing a cross section of the Enterprise covers the entire northern wall")
(test-e2c 'Area1014 "The room is very neat and tidy, with a couch and several chairs aranged around a coffee table by the eastern wall")
(test-e2c 'Area1022 "You're in the main shuttle bay of the Enterprise")
(test-e2c 'Area1038 "This is where the ship's pilot and information officer sit")
(test-e2c 'NpcCol1000-Geordi684 "Lieutenant Commander Geordi LaForge is standing here")
(test-e2c 'Area1038 "This is where the Captain, Commander Riker, and Counselor Troi sit")
(test-e2c 'Area1029 "A strange looking machine sits in the middle of the room, up on a slightly raised platform")
(test-e2c 'Area1034 "From here, you can access the other decks on the Enterprise")
(test-e2c 'Area1010 "Several vertical rows of lights make this place very well lit")
(test-e2c 'Area1036 "You find yourself on the upper half of the main bridge of the USS Enterprise")
(test-e2c 'Area1006 "A computer terminal is sitting near the southern wall, where the transporter chief can control the transporters")
(test-e2c 'Area1011 "Several vertical rows of lights make this place very well lit")
(test-e2c 'Area1038 "Two computer consoles with built in chairs rest about ten feet in front of the chairs, also facing the view screen")
(test-e2c 'Area1021 "You're now in Ten Forward, the entertainment room of the Enterprise")
(test-e2c 'Area1038 "Three large chairs in the northern part of the room, in front of the railing, face the screen")
(test-e2c 'Area1038 "An enormous view screen covers almost the entire northern wall, and is currently displaying a view of the stars rushing by")
(test-e2c 'Area1018 "Several different paintings are attached to the walls, and you also notice a few sculptures")
(test-e2c 'Area1003 "In front of the screen is a large desk, which is covered in computer controls")
(test-e2c 'Area1014 "A small partition at the northern part of the room seperates his sleeping area with the rest of the room")
(test-e2c 'Area1022 "It's quite a large room, with a very high ceiling and a lot of floor space")
(test-e2c 'NpcCol1003-Dr-Crusher677 "Lieutenant Beverly Crusher is here, looking for someone to heal")
(test-e2c 'Area1037 "The entire eastern wall is covered with windows, looking out into space")
(test-e2c 'Area1035 "A small computer screen is sitting on the desk, as well as several other papers and documents")
(test-e2c 'Area1029 "It looks as though something (or someone) could be placed inside, hooked up to the multitude of wires and cables, and have scientific tests performed on it(or them)")
(test-e2c 'Area1000 "You find yourself in the middle of main engineering")
(test-e2c 'Area1010 "From here, you can access the other decks on the Enterprise")
(test-e2c 'Area1036 "Directly in front of you is a thick railing that contains many different computer panels used for the tactical systems of the ship")
(test-e2c 'Area1006 "Eight round transport pads have been arranged in a circle, on a raised platform against the northern wall")
(test-e2c 'Area1030 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1017 "You emerge into a dark narrow alley")
(test-e2c 'Area1011 "From here, you can accessthe other decks on the Enterprise")
(test-e2c 'Area1005 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1021 "The entire northern wall is covered with windows looking out into space, while two large wooden doors with the Starfleet insignia stamped on them face south")
(test-e2c 'Area1007 "You step through the doors and find yourself in a large school room")
(test-e2c 'Area1016 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1003 "You can't see a bed in this room, but you figure it's because Data doesn't sleep")
(test-e2c 'NpcCol1012-Ensign752 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1022 "You can see three different shuttle crafts sitting here, waiting to be flown")
(test-e2c 'Area1024 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1040 "You can see stars in every direction, which provide the only light here")
(test-e2c 'Area1013 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1029 "A complex looking computer console is facing this machine")
(test-e2c 'Area1000 "The room is longer than it is wide, and it has fairly low ceilings")
(test-e2c 'Area1035 "You're standing in Captain Picard's ready room")
(test-e2c 'Area1010 "")
(test-e2c 'Area1036 "The entire southern wall is covered with computer consoles, where the ship's main systems are controlled")
(test-e2c 'Area1030 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1001 "You're in the middle of Geordi's quarters")
(test-e2c 'Area1017 "Tall dark brick buildings block your way north and south")
(test-e2c 'NpcCol1012-Ensign736 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1005 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1021 "Many round metal tables are scattered around the room, surrounded by metal chairs")
(test-e2c 'Area1007 "Various tables and chairs are set up all around the room, and many paintings and drawings have been attached to the walls")
(test-e2c 'Area1016 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1018 "A neatly made bed is located by the northern wall, in between two large windows looking out into space")
(test-e2c 'Area1022 "A large grey door leads into space")
(test-e2c 'Area1001 "The room is sparsely decorated, due to the fact that Geordi is blind")
(test-e2c 'Area1040 "You feel very cold")
(test-e2c 'Area1013 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1035 "A single high window beside the desk looks into space, and a fish tank is located in the northwestern corner of the room")
(test-e2c 'Area1000 "Computer terminals cover all the walls, and a large table built into the floor sits in the middle of the room")
(test-e2c 'Area1035 "A long couch has been placed beside the door, while a large U shaped desk is located by the northern wall")
(test-e2c 'Area1036 "Two small curved ramps on either side of the room lead north to the lower part of the bridge, and a large circular skylight shows the space outside the ship")
(test-e2c 'Area1025 "You're in Worf's quarters")
(test-e2c 'Area1024 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1017 "You can see that the windows on the buildings are fairly high, and some have been boarded up")
(test-e2c 'Area1005 "You notice a tiny computer panel embedded into the wall")
(test-e2c 'Area1021 "A long bar spans almost the entire length of the southern part of the room, and about two dozen bar stools are sitting in front of it")
(test-e2c 'Area1007 "Several computer consoles with a children's interface on them can be seen on the tables")
(test-e2c 'Area1016 "You see the holodeck's control panel beside the holodeck door, and it has some information on it")
(test-e2c 'Area1001 "A small personal computer sits on a desk against the western wall, in between two windows that look out into space")
(test-e2c 'Area1009 "You're now on Holodeck 2")
(test-e2c 'NpcCol1013-Alexander671 "Alexander Rozhenko is here, practicing laughing hour")
(test-e2c 'Area1013 "You notice a tiny computer panel embedded into the wall")
(test-e2c 'Area1000 "At the far end of the room you see the warp core, a large pulsating vertical tube")
(test-e2c 'Area1041 "You're floating in outer space right above the USS Enterprise")
(test-e2c 'Area1025 "A small table is sitting in the southeastern corner, and on it is a small potted plant")
(test-e2c 'Area1024 "You notice a tiny computer panel embedded into the wall")
(test-e2c 'Area1017 "The smell from the rotting food and garbage mixing with the foul water on the ground is unbearable")
(test-e2c 'NpcCol1012-Ensign744 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1021 "It's very noisy in here, due to all the talking and laughing")
(test-e2c 'NpcCol1012-Ensign740 "A nervous looking ensign is standing here, watching you")
(test-e2c 'Area1032 "From here, you can access the other decks on the Enterprise")
(test-e2c 'Area1041 "You feel very cold")
(test-e2c 'Area1041 "You can see stars in every direction, which provide the only light here")
(test-e2c 'Area1028 "You find yourself standing by the door of Captain Picard's quarters")
(test-e2c 'Area1026 "You emerge into the Enterprise gym")
(test-e2c 'Area1033 "You're in the turbolift")
(test-e2c 'Area1009 "The room is just a large cube, with jet black walls and a yellow grid painted on the floors, the walls, and the ceiling")
(test-e2c 'NpcCol1011-Spot727 "Spot, Data's pet cat, is sitting here looking at you")
(test-e2c 'Area1042 "All you can see is blue flashing light")
(test-e2c 'Area1031 "You're in the cargo bay 2 of the Enterprise")
(test-e2c 'Area1028 "You can see several different ancient artifacts on tables and small pedestals, and a large wooden wardrobe is facing south")
(test-e2c 'Area1004 "You're in the dimly lit Brig")
(test-e2c 'Area1025 "An impressive selection of Klingon weapons have been mounted on the northern wall, and a partition splits this room with Worf's bedroom to the east")
(test-e2c 'Area1029 "Around the rest of the room are countertops with with the odd computer terminal")
(test-e2c 'Area1001 "A neatly made bed has been placed against the northern wall")
(test-e2c 'Area1017 "You can hear the sounds of a bustling marketpace to the east")
(test-e2c 'NpcCol1010-Livingston726 "Livingston the fish is here, swimming about in his tank")
(test-e2c 'NpcCol1012-Ensign728 "A nervous looking ensign is standing here, watching you")
(test-e2c 'NpcCol1005-Riker707 "Commander William Riker is here, staring at you")
(test-e2c 'Area1029 "You're in the Enterprise science lab")
(test-e2c 'Area1019 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1028 "He isn't very fond of visitors, but you decide to stay and have a look around")
(test-e2c 'Area1032 "You're in the turbolift")
(test-e2c 'Area1026 "The room is quite large, with a soft grey floor")
(test-e2c 'Area1033 "The turbolift walls have been rounded off, making it in the shape of a tube")
(test-e2c 'Area1009 "This is where different programs can be loaded and experienced, which seem totally real")
(test-e2c 'NpcCol1009-Wesley716 "Wesley Crusher is here, eagerly trying to earn your praise")
(test-e2c 'Area1042 "As you try to look down at your body, you realize that there's nothing there!")
(test-e2c 'Area1042 "It feels as though your body is racing around at high speeds")
(test-e2c 'Area1035 "This is where the Captain makes all of his important decisions")
(test-e2c 'Area1028 "A comfortable looking recliner with a matching footrest sits beside the door, along with a bright reading lamp and end table")
(test-e2c 'Area1004 "This is where all the criminals and prisoners are kept while on board the Enterprise")
(test-e2c 'Area1020 "You're standing in the dimly lit Enterprise Security")
(test-e2c 'Area1034 "The turbolift walls have been rounded off, making it in the shape of a tube")
(test-e2c 'Area1015 "You're in the middle of the Enterprise's Sick Bay")
(test-e2c 'Area1017 "The archway leading out of the holodeck is west")
(test-e2c 'NpcCol1008-OBrien696 "Chief O'Brien is here, waiting to teleport you somwhere")
(test-e2c 'Area1008 "You find yourself in the middle of a well lit corridor on the Enterprise")
(test-e2c 'Area1039 "You can see stars in every direction, which provide the only light here")
(test-e2c 'Area1019 "It isn't very wide, and the light beige walls have been rounded, making the corridor an oval shape")
(test-e2c 'Area1012 "You're in the main cargo bay of the Enterprise")
(test-e2c 'Area1032 "The turbolift walls have been rounded off, making it in the shape of a tube")
(test-e2c 'Area1026 "A set of lockers against the southern wall contain all of the necessary equipment needed for using the gym")
(test-e2c 'Area1033 "Several vertical rows of lights make this place very well lit")
(test-e2c 'Area1009 "Right now, this holodeck is not functioning")
(test-e2c 'Area1042 "You find yourself in a transporter beam")
(test-e2c 'Area1023 "You're in Counselor Deanna Troi's quarters")
(test-e2c 'Area1031 "It's quite a large room, with a very high ceiling and a lot of floor space")
(test-e2c 'Area1004 "Three fairly large cells can been seen in the southern part of the room, and they're all empty")
(test-e2c 'Area1020 "Weapons lockers cover all of the walls, except along the northern wall, where a large glass window protects dozens of different phasors, blaster rifles, and other high tech weapons")
(test-e2c 'Area1037 "You're in the conference room of the Enterprise")
(test-e2c 'Area1015 "About a dozen beds are arranged along the walls of the room, while several carts covered with medical supplies are scattered around the room")


;;testNumbers:-
;; ( PRINT "NUMBERS")

(test-e2c 1 "2 million attended ")
(test-e2c 1 "2863764 attended")
(test-e2c 1 "2 million people attended")
(test-e2c 2 "2 million person attended ")
(test-e2c 1 "a million attended ")
(test-e2c 1 "a million people attended ")
(test-e2c 1 "about 2 million people attended")
(test-e2c 1 "about 2 million attended")
(test-e2c 2 "about people attended")
(test-e2c 1 "a million such people attended")
(test-e2c 2 "a million such attended")
(test-e2c 1 "5 million of the people attended")
(test-e2c 1 "5 thousand invited by Bob attended")
(test-e2c 1 "The 5 thousand invited by Bob attended")
(test-e2c 1 "The thousands of people who attended enjoyed it")
(test-e2c 1 "The 5 thousand people invited by Bob attended")
(test-e2c 1 "The nearly 5 million people who attended enjoyed it")
(test-e2c 1 "a few attended")
(test-e2c 1 "a few million people attended")
(test-e2c 1 "a few people attended")
(test-e2c 1 "few attended")
(test-e2c 2 "few million people attended")
(test-e2c 1 "millions attended")
(test-e2c 2 "5 millions attended")
(test-e2c 1 "millions of people attended")
(test-e2c 1 "hundreds of millions of people attended")
(test-e2c 1 "5 million years ago, the earth was covered with ice")
(test-e2c 1 "millions of years ago, the earth was covered with ice")
(test-e2c 2 "dogs of years ago, the earth was covered with ice")
(test-e2c 2 "the five million years ago, the earth was covered with ice")
(test-e2c 2 "the other five million years ago, the earth was covered with ice")
(test-e2c 2 "5 million ago, the earth was covered with ice")
(test-e2c 1 "The city of New York contains over one hundred million billion brain cells")
(test-e2c 1 "Almost one third of the people in the country have no health insurance")
(test-e2c 1 "Of all the people in this country, almost one third have no health insurance")
(test-e2c 1 "Three quarters of a million people in this city have no health insurance")
(test-e2c 1 "The price of the stock rose three tenths of one point")
(test-e2c 2 "The price of the stock rose three tenths of one dog")
(test-e2c 1 "The nearest drug store is about three quarters of a mile away ")
(test-e2c 1 "The nearest drug store is about 3/4 of a mile away ")
(test-e2c 1 "Every morning I walk 3 1/2 miles")
(test-e2c 2 "Every morning I walk 1/2 3 miles")
(test-e2c 1 "Nearly 1/2 million people work here every day")
(test-e2c 1 "Nearly 1 1/4 million people work here every day")
(test-e2c 2 "Nearly 1 million 1/4 people work here every day")
(test-e2c 1 "Between 500 and 1000 people pass through here every minute")
(test-e2c 1 "They are the Number 3 auto maker and a Fortune 500 company")
(test-e2c 1 "I live at 805 West Indiana Street")
